/*
 *  arch/arm/mach-footbridge/include/mach/vmalloc.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */


#define VMALLOC_END       (PAGE_OFFSET + 0x30000000)
