/*
 * Merisc exports
 *
 * Copyright (C) 2008 Martinsson Elektronik AB
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef __ARCH_AVR32_BOARDS_MERISC_MERISC_H
#define __ARCH_AVR32_BOARDS_MERISC_MERISC_H

const char *merisc_revision(void);
const char *merisc_model(void);

extern struct class merisc_class;

#endif /* __ARCH_AVR32_BOARDS_MERISC_MERISC_H */
