#ifndef __ASM_SH_CPU_SH5_ADDRSPACE_H
#define __ASM_SH_CPU_SH5_ADDRSPACE_H

#define	PHYS_PERIPHERAL_BLOCK	0x09000000
#define PHYS_DMAC_BLOCK		0x0e000000
#define PHYS_PCI_BLOCK		0x60000000
#define PHYS_EMI_BLOCK		0xff000000

/* No segmentation.. */

#endif /* __ASM_SH_CPU_SH5_ADDRSPACE_H */
