#ifndef __UM_PROCESSOR_PPC_H
#define __UM_PROCESSOR_PPC_H

#if defined(__ASSEMBLY__)

#define CONFIG_PPC_MULTIPLATFORM
#include "arch/processor.h"

#else

#include "asm/processor-generic.h"

#endif

#endif
