/*
 * Copyright 2003 PathScale, Inc.
 *
 * Licensed under the GPL
 */

#ifndef __UM_ARCHPARAM_X86_64_H
#define __UM_ARCHPARAM_X86_64_H


/* No user-accessible fixmap addresses, i.e. vsyscall */
#define FIXADDR_USER_START	0
#define FIXADDR_USER_END	0

#endif

