/*
 * This is the place to register all trace points as events.
 */

#include <linux/stringify.h>

#include <trace/trace_events.h>

#include "trace_output.h"

#include "trace_events_stage_1.h"
#include "trace_events_stage_2.h"
#include "trace_events_stage_3.h"

