/*
 * Copyright (C) 2008 Michal Simek <monstr@monstr.eu>
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#ifndef _ASM_MICROBLAZE_LMB_H
#define _ASM_MICROBLAZE_LMB_H

/* LMB limit is OFF */
#define LMB_REAL_LIMIT	0xFFFFFFFF

#endif /* _ASM_MICROBLAZE_LMB_H */


