/* Autogenerated Changes here will be lost!
 * generated by ../gen_sw.pl Mon Apr 11 16:10:18 2005 iop_sw.cfg
 */
#define iop_version 0
#define iop_fifo_in0_extra 64
#define iop_fifo_in1_extra 128
#define iop_fifo_out0_extra 192
#define iop_fifo_out1_extra 256
#define iop_trigger_grp0 320
#define iop_trigger_grp1 384
#define iop_trigger_grp2 448
#define iop_trigger_grp3 512
#define iop_trigger_grp4 576
#define iop_trigger_grp5 640
#define iop_trigger_grp6 704
#define iop_trigger_grp7 768
#define iop_crc_par0 896
#define iop_crc_par1 1024
#define iop_dmc_in0 1152
#define iop_dmc_in1 1280
#define iop_dmc_out0 1408
#define iop_dmc_out1 1536
#define iop_fifo_in0 1664
#define iop_fifo_in1 1792
#define iop_fifo_out0 1920
#define iop_fifo_out1 2048
#define iop_scrc_in0 2176
#define iop_scrc_in1 2304
#define iop_scrc_out0 2432
#define iop_scrc_out1 2560
#define iop_timer_grp0 2688
#define iop_timer_grp1 2816
#define iop_timer_grp2 2944
#define iop_timer_grp3 3072
#define iop_sap_in 3328
#define iop_sap_out 3584
#define iop_spu0 3840
#define iop_spu1 4096
#define iop_sw_cfg 4352
#define iop_sw_cpu 4608
#define iop_sw_mpu 4864
#define iop_sw_spu0 5120
#define iop_sw_spu1 5376
#define iop_mpu 5632
